/******************************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2014 Crytek
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 ******************************************************************************/


#pragma once

#include "gl_common.h"

struct GLHookSet
{
	// ++ dllexport
	PFNGLBINDTEXTUREPROC glBindTexture;
	PFNGLBLENDFUNCPROC glBlendFunc;
	PFNGLCLEARPROC glClear;
	PFNGLCLEARCOLORPROC glClearColor;
	PFNGLCLEARDEPTHPROC glClearDepth;
	PFNGLCOLORMASKPROC glColorMask;
	PFNGLCULLFACEPROC glCullFace;
	PFNGLDEPTHFUNCPROC glDepthFunc;
	PFNGLDEPTHMASKPROC glDepthMask;
	PFNGLSTENCILFUNCPROC glStencilFunc;
	PFNGLSTENCILMASKPROC glStencilMask;
	PFNGLSTENCILOPPROC glStencilOp;
	PFNGLDISABLEPROC glDisable;
	PFNGLDRAWBUFFERPROC glDrawBuffer;
	PFNGLDRAWELEMENTSPROC glDrawElements;
	PFNGLDRAWARRAYSPROC glDrawArrays;
	PFNGLENABLEPROC glEnable;
	PFNGLFLUSHPROC glFlush;
	PFNGLFINISHPROC glFinish;
	PFNGLFRONTFACEPROC glFrontFace;
	PFNGLGENTEXTURESPROC glGenTextures;
	PFNGLDELETETEXTURESPROC glDeleteTextures;
	PFNGLGETERRORPROC glGetError;
	PFNGLISENABLEDPROC glIsEnabled;
	PFNGLGETTEXLEVELPARAMETERIVPROC glGetTexLevelParameteriv;
	PFNGLGETTEXLEVELPARAMETERFVPROC glGetTexLevelParameterfv;
	PFNGLGETTEXPARAMETERFVPROC glGetTexParameterfv;
	PFNGLGETTEXPARAMETERIVPROC glGetTexParameteriv;
	PFNGLGETTEXIMAGEPROC glGetTexImage;
	PFNGLGETBOOLEANVPROC glGetBooleanv;
	PFNGLGETFLOATVPROC glGetFloatv;
	PFNGLGETDOUBLEVPROC glGetDoublev;
	PFNGLGETINTEGERVPROC glGetIntegerv;
	PFNGLGETSTRINGPROC glGetString;
	PFNGLHINTPROC glHint;
	PFNGLPIXELSTOREIPROC glPixelStorei;
	PFNGLPIXELSTOREFPROC glPixelStoref;
	PFNGLPOLYGONMODEPROC glPolygonMode;
	PFNGLPOLYGONOFFSETPROC glPolygonOffset;
	PFNGLREADPIXELSPROC glReadPixels;
	PFNGLREADBUFFERPROC glReadBuffer;
	PFNGLSCISSORPROC glScissor;
	PFNGLTEXIMAGE1DPROC glTexImage1D;
	PFNGLTEXIMAGE2DPROC glTexImage2D;
	PFNGLTEXIMAGE3DPROC glTexImage3D;
	PFNGLTEXSUBIMAGE1DPROC glTexSubImage1D;
	PFNGLTEXSUBIMAGE2DPROC glTexSubImage2D;
	PFNGLTEXPARAMETERFPROC glTexParameterf;
	PFNGLTEXPARAMETERFVPROC glTexParameterfv;
	PFNGLTEXPARAMETERIPROC glTexParameteri;
	PFNGLTEXPARAMETERIVPROC glTexParameteriv;
	PFNGLVIEWPORTPROC glViewport;
	// --

	// ++ gl
	PFNGLACTIVETEXTUREPROC glActiveTexture;
	PFNGLTEXSTORAGE1DPROC glTexStorage1D;
	PFNGLTEXSTORAGE2DPROC glTexStorage2D;
	PFNGLTEXSTORAGE3DPROC glTexStorage3D;
	PFNGLTEXSUBIMAGE3DPROC glTexSubImage3D;
	PFNGLCOMPRESSEDTEXIMAGE1DPROC glCompressedTexImage1D;
	PFNGLCOMPRESSEDTEXIMAGE2DPROC glCompressedTexImage2D;
	PFNGLCOMPRESSEDTEXIMAGE3DPROC glCompressedTexImage3D;
	PFNGLCOMPRESSEDTEXSUBIMAGE1DPROC glCompressedTexSubImage1D;
	PFNGLCOMPRESSEDTEXSUBIMAGE2DPROC glCompressedTexSubImage2D;
	PFNGLCOMPRESSEDTEXSUBIMAGE3DPROC glCompressedTexSubImage3D;
	PFNGLTEXBUFFERRANGEPROC glTexBufferRange;
	PFNGLTEXTUREVIEWPROC glTextureView;
	PFNGLGENERATEMIPMAPPROC glGenerateMipmap;
	PFNGLCOPYIMAGESUBDATAPROC glCopyImageSubData;
	PFNGLGETINTERNALFORMATIVPROC glGetInternalformativ;
	PFNGLGETINTERNALFORMATI64VPROC glGetInternalformati64v;
	PFNGLGETBUFFERPARAMETERIVPROC glGetBufferParameteriv;
	PFNGLGETSTRINGIPROC glGetStringi;
	PFNGLGETBOOLEANI_VPROC glGetBooleani_v;
	PFNGLGETINTEGERI_VPROC glGetIntegeri_v;
	PFNGLGETFLOATI_VPROC glGetFloati_v;
	PFNGLGETDOUBLEI_VPROC glGetDoublei_v;
	PFNGLGETINTEGER64I_VPROC glGetInteger64i_v;
	PFNGLCHECKFRAMEBUFFERSTATUSPROC glCheckFramebufferStatus;
	PFNGLBLENDCOLORPROC glBlendColor;
	PFNGLBLENDFUNCIPROC glBlendFunci;
	PFNGLBLENDFUNCSEPARATEPROC glBlendFuncSeparate;
	PFNGLBLENDFUNCSEPARATEIPROC glBlendFuncSeparatei;
	PFNGLBLENDEQUATIONSEPARATEPROC glBlendEquationSeparate;
	PFNGLBLENDEQUATIONSEPARATEIPROC glBlendEquationSeparatei;
	PFNGLSTENCILFUNCSEPARATEPROC glStencilFuncSeparate;
	PFNGLSTENCILMASKSEPARATEPROC glStencilMaskSeparate;
	PFNGLSTENCILOPSEPARATEPROC glStencilOpSeparate;
	PFNGLCOLORMASKIPROC glColorMaski;
	PFNGLSAMPLEMASKIPROC glSampleMaski;
	PFNGLDEPTHRANGEPROC glDepthRange;
	PFNGLDEPTHRANGEFPROC glDepthRangef;
	PFNGLDEPTHRANGEARRAYVPROC glDepthRangeArrayv;
	PFNGLDEPTHBOUNDSEXTPROC glDepthBoundsEXT;
	PFNGLCREATESHADERPROC glCreateShader;
	PFNGLDELETESHADERPROC glDeleteShader;
	PFNGLSHADERSOURCEPROC glShaderSource;
	PFNGLCOMPILESHADERPROC glCompileShader;
	PFNGLCREATESHADERPROGRAMVPROC glCreateShaderProgramv;
	PFNGLGETSHADERIVPROC glGetShaderiv;
	PFNGLGETSHADERINFOLOGPROC glGetShaderInfoLog;
	PFNGLCREATEPROGRAMPROC glCreateProgram;
	PFNGLDELETEPROGRAMPROC glDeleteProgram;
	PFNGLATTACHSHADERPROC glAttachShader;
	PFNGLDETACHSHADERPROC glDetachShader;
	PFNGLRELEASESHADERCOMPILERPROC glReleaseShaderCompiler;
	PFNGLLINKPROGRAMPROC glLinkProgram;
	PFNGLPROGRAMPARAMETERIPROC glProgramParameteri;
	PFNGLPROGRAMUNIFORM1IPROC glProgramUniform1i;
	PFNGLPROGRAMUNIFORM1FVPROC glProgramUniform1fv;
	PFNGLPROGRAMUNIFORM1IVPROC glProgramUniform1iv;
	PFNGLPROGRAMUNIFORM1UIVPROC glProgramUniform1uiv;
	PFNGLPROGRAMUNIFORM2FVPROC glProgramUniform2fv;
	PFNGLPROGRAMUNIFORM3FVPROC glProgramUniform3fv;
	PFNGLPROGRAMUNIFORM4FVPROC glProgramUniform4fv;
	PFNGLUSEPROGRAMPROC glUseProgram;
	PFNGLUSEPROGRAMSTAGESPROC glUseProgramStages;
	PFNGLVALIDATEPROGRAMPROC glValidateProgram;
	PFNGLGETPROGRAMIVPROC glGetProgramiv;
	PFNGLGETPROGRAMINFOLOGPROC glGetProgramInfoLog;
	PFNGLGETPROGRAMINTERFACEIVPROC glGetProgramInterfaceiv;
	PFNGLGETPROGRAMRESOURCEINDEXPROC glGetProgramResourceIndex;
	PFNGLGETPROGRAMRESOURCEIVPROC glGetProgramResourceiv;
	PFNGLGETPROGRAMRESOURCENAMEPROC glGetProgramResourceName;
	PFNGLGENPROGRAMPIPELINESPROC glGenProgramPipelines;
	PFNGLBINDPROGRAMPIPELINEPROC glBindProgramPipeline;
	PFNGLDELETEPROGRAMPIPELINESPROC glDeleteProgramPipelines;
	PFNGLGETPROGRAMPIPELINEIVPROC glGetProgramPipelineiv;
	PFNGLGETPROGRAMPIPELINEINFOLOGPROC glGetProgramPipelineInfoLog;
	PFNGLVALIDATEPROGRAMPIPELINEPROC glValidateProgramPipeline;
	PFNGLDEBUGMESSAGECALLBACKPROC glDebugMessageCallback; // aliases glDebugMessageCallbackARB
	PFNGLDEBUGMESSAGECONTROLPROC glDebugMessageControl;
	PFNGLDEBUGMESSAGEINSERTPROC glDebugMessageInsert;
	PFNGLPUSHDEBUGGROUPPROC glPushDebugGroup;
	PFNGLPOPDEBUGGROUPPROC glPopDebugGroup;
	PFNGLGETOBJECTLABELPROC glGetObjectLabel;
	PFNGLOBJECTLABELPROC glObjectLabel;
	PFNGLENABLEIPROC glEnablei;
	PFNGLDISABLEIPROC glDisablei;
	PFNGLISENABLEDIPROC glIsEnabledi;
	PFNGLGENBUFFERSPROC glGenBuffers;
	PFNGLBINDBUFFERPROC glBindBuffer;
	PFNGLDRAWBUFFERSPROC glDrawBuffers;
	PFNGLGENFRAMEBUFFERSPROC glGenFramebuffers;
	PFNGLBINDFRAMEBUFFERPROC glBindFramebuffer;
	PFNGLFRAMEBUFFERTEXTUREPROC glFramebufferTexture;
	PFNGLFRAMEBUFFERTEXTURE2DPROC glFramebufferTexture2D;
	PFNGLFRAMEBUFFERTEXTURELAYERPROC glFramebufferTextureLayer;
	PFNGLDELETEFRAMEBUFFERSPROC glDeleteFramebuffers;
	PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC glGetFramebufferAttachmentParameteriv;
	PFNGLFENCESYNCPROC glFenceSync;
	PFNGLCLIENTWAITSYNCPROC glClientWaitSync;
	PFNGLWAITSYNCPROC glWaitSync;
	PFNGLDELETESYNCPROC glDeleteSync;
	PFNGLGENQUERIESPROC glGenQueries;
	PFNGLBEGINQUERYPROC glBeginQuery;
	PFNGLENDQUERYPROC glEndQuery;
	PFNGLGETQUERYOBJECTUI64VPROC glGetQueryObjectui64v;
	PFNGLGETQUERYOBJECTUIVPROC glGetQueryObjectuiv;
	PFNGLDELETEQUERIESPROC glDeleteQueries;
	PFNGLBUFFERDATAPROC glBufferData;
	PFNGLBUFFERSTORAGEPROC glBufferStorage;
	PFNGLBUFFERSUBDATAPROC glBufferSubData;
	PFNGLCOPYBUFFERSUBDATAPROC glCopyBufferSubData;
	PFNGLBINDBUFFERBASEPROC glBindBufferBase;
	PFNGLBINDBUFFERRANGEPROC glBindBufferRange;
	PFNGLMAPBUFFERPROC glMapBuffer;
	PFNGLMAPBUFFERRANGEPROC glMapBufferRange;
	PFNGLUNMAPBUFFERPROC glUnmapBuffer;
	PFNGLDELETEBUFFERSPROC glDeleteBuffers;
	PFNGLGETBUFFERSUBDATAPROC glGetBufferSubData;
	PFNGLGENVERTEXARRAYSPROC glGenVertexArrays;
	PFNGLBINDVERTEXARRAYPROC glBindVertexArray;
	PFNGLDELETEVERTEXARRAYSPROC glDeleteVertexArrays;
	PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointer;
	PFNGLVERTEXATTRIBIPOINTERPROC glVertexAttribIPointer;
	PFNGLVERTEXATTRIBBINDINGPROC glVertexAttribBinding;
	PFNGLVERTEXATTRIBFORMATPROC glVertexAttribFormat;
	PFNGLVERTEXATTRIBIFORMATPROC glVertexAttribIFormat;
	PFNGLBINDATTRIBLOCATIONPROC glBindAttribLocation;
	PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArray;
	PFNGLDISABLEVERTEXATTRIBARRAYPROC glDisableVertexAttribArray;
	PFNGLGETVERTEXATTRIBIVPROC glGetVertexAttribiv;
	PFNGLGETVERTEXATTRIBPOINTERVPROC glGetVertexAttribPointerv;
	PFNGLBINDVERTEXBUFFERPROC glBindVertexBuffer;
	PFNGLVERTEXBINDINGDIVISORPROC glVertexBindingDivisor;
	PFNGLGETCOMPRESSEDTEXIMAGEPROC glGetCompressedTexImage;
	PFNGLGENSAMPLERSPROC glGenSamplers;
	PFNGLBINDSAMPLERPROC glBindSampler;
	PFNGLDELETESAMPLERSPROC glDeleteSamplers;
	PFNGLSAMPLERPARAMETERIPROC glSamplerParameteri;
	PFNGLSAMPLERPARAMETERFPROC glSamplerParameterf;
	PFNGLSAMPLERPARAMETERIVPROC glSamplerParameteriv;
	PFNGLSAMPLERPARAMETERFVPROC glSamplerParameterfv;
	PFNGLSAMPLERPARAMETERIIVPROC glSamplerParameterIiv;
	PFNGLSAMPLERPARAMETERIUIVPROC glSamplerParameterIuiv;
	PFNGLPATCHPARAMETERIPROC glPatchParameteri;
	PFNGLPATCHPARAMETERFVPROC glPatchParameterfv;
	PFNGLCLEARBUFFERFVPROC glClearBufferfv;
	PFNGLCLEARBUFFERIVPROC glClearBufferiv;
	PFNGLCLEARBUFFERUIVPROC glClearBufferuiv;
	PFNGLCLEARBUFFERFIPROC glClearBufferfi;
	PFNGLSCISSORARRAYVPROC glScissorArrayv;
	PFNGLSCISSORINDEXEDPROC glScissorIndexed;
	PFNGLSCISSORINDEXEDVPROC glScissorIndexedv;
	PFNGLVIEWPORTINDEXEDFPROC glViewportIndexedf;
	PFNGLVIEWPORTINDEXEDFVPROC glViewportIndexedfv;
	PFNGLVIEWPORTARRAYVPROC glViewportArrayv;
	PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocation;
	PFNGLGETUNIFORMINDICESPROC glGetUniformIndices;
	PFNGLGETUNIFORMBLOCKINDEXPROC glGetUniformBlockIndex;
	PFNGLGETATTRIBLOCATIONPROC glGetAttribLocation;
	PFNGLGETACTIVEUNIFORMPROC glGetActiveUniform;
	PFNGLGETACTIVEUNIFORMBLOCKIVPROC glGetActiveUniformBlockiv;
	PFNGLGETACTIVEUNIFORMSIVPROC glGetActiveUniformsiv;
	PFNGLGETACTIVEATTRIBPROC glGetActiveAttrib;
	PFNGLGETUNIFORMFVPROC glGetUniformfv;
	PFNGLGETUNIFORMIVPROC glGetUniformiv;
	PFNGLGETUNIFORMUIVPROC glGetUniformuiv;
	PFNGLGETUNIFORMDVPROC glGetUniformdv;
	PFNGLUNIFORMBLOCKBINDINGPROC glUniformBlockBinding;
	PFNGLUNIFORMMATRIX4FVPROC glUniformMatrix4fv;
	PFNGLUNIFORM1FPROC glUniform1f;
	PFNGLUNIFORM1IPROC glUniform1i;
	PFNGLUNIFORM1UIPROC glUniform1ui;
	PFNGLUNIFORM1FVPROC glUniform1fv;
	PFNGLUNIFORM1IVPROC glUniform1iv;
	PFNGLUNIFORM1UIVPROC glUniform1uiv;
	PFNGLUNIFORM2FVPROC glUniform2fv;
	PFNGLUNIFORM3FVPROC glUniform3fv;
	PFNGLUNIFORM4FVPROC glUniform4fv;
	PFNGLDRAWRANGEELEMENTSPROC glDrawRangeElements;
	PFNGLDRAWARRAYSINSTANCEDBASEINSTANCEPROC glDrawArraysInstancedBaseInstance;
	PFNGLDRAWARRAYSINSTANCEDPROC glDrawArraysInstanced;
	PFNGLDRAWELEMENTSINSTANCEDPROC glDrawElementsInstanced;
	PFNGLDRAWELEMENTSINSTANCEDBASEINSTANCEPROC glDrawElementsInstancedBaseInstance;
	PFNGLDRAWELEMENTSBASEVERTEXPROC glDrawElementsBaseVertex;
	PFNGLDRAWELEMENTSINSTANCEDBASEVERTEXPROC glDrawElementsInstancedBaseVertex;
	PFNGLDRAWELEMENTSINSTANCEDBASEVERTEXBASEINSTANCEPROC glDrawElementsInstancedBaseVertexBaseInstance;
	PFNGLBLITFRAMEBUFFERPROC glBlitFramebuffer;
	PFNGLCHECKNAMEDFRAMEBUFFERSTATUSEXTPROC glCheckNamedFramebufferStatusEXT;
	PFNGLCOMPRESSEDTEXTUREIMAGE1DEXTPROC glCompressedTextureImage1DEXT;
	PFNGLCOMPRESSEDTEXTUREIMAGE2DEXTPROC glCompressedTextureImage2DEXT;
	PFNGLCOMPRESSEDTEXTUREIMAGE3DEXTPROC glCompressedTextureImage3DEXT;
	PFNGLCOMPRESSEDTEXTURESUBIMAGE1DEXTPROC glCompressedTextureSubImage1DEXT;
	PFNGLCOMPRESSEDTEXTURESUBIMAGE2DEXTPROC glCompressedTextureSubImage2DEXT;
	PFNGLCOMPRESSEDTEXTURESUBIMAGE3DEXTPROC glCompressedTextureSubImage3DEXT;
	PFNGLFRAMEBUFFERDRAWBUFFERSEXTPROC glFramebufferDrawBuffersEXT;
	PFNGLGENERATETEXTUREMIPMAPEXTPROC glGenerateTextureMipmapEXT;
	PFNGLGETCOMPRESSEDTEXTUREIMAGEEXTPROC glGetCompressedTextureImageEXT;
	PFNGLGETNAMEDBUFFERSUBDATAEXTPROC glGetNamedBufferSubDataEXT;
	PFNGLGETNAMEDBUFFERPARAMETERIVEXTPROC glGetNamedBufferParameterivEXT;
	PFNGLGETNAMEDFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC glGetNamedFramebufferAttachmentParameterivEXT;
	PFNGLGETTEXTUREIMAGEEXTPROC glGetTextureImageEXT;
	PFNGLGETTEXTURELEVELPARAMETERIVEXTPROC glGetTextureLevelParameterivEXT;
	PFNGLMAPNAMEDBUFFEREXTPROC glMapNamedBufferEXT;
	PFNGLMAPNAMEDBUFFERRANGEEXTPROC glMapNamedBufferRangeEXT;
	PFNGLNAMEDBUFFERDATAEXTPROC glNamedBufferDataEXT;
	PFNGLNAMEDBUFFERSTORAGEEXTPROC glNamedBufferStorageEXT;
	PFNGLNAMEDBUFFERSUBDATAEXTPROC glNamedBufferSubDataEXT;
	PFNGLNAMEDCOPYBUFFERSUBDATAEXTPROC glNamedCopyBufferSubDataEXT;
	PFNGLNAMEDFRAMEBUFFERTEXTUREEXTPROC glNamedFramebufferTextureEXT;
	PFNGLNAMEDFRAMEBUFFERTEXTURE2DEXTPROC glNamedFramebufferTexture2DEXT;
	PFNGLNAMEDFRAMEBUFFERTEXTURELAYEREXTPROC glNamedFramebufferTextureLayerEXT;
	PFNGLTEXTUREBUFFERRANGEEXTPROC glTextureBufferRangeEXT;
	PFNGLTEXTUREIMAGE1DEXTPROC glTextureImage1DEXT;
	PFNGLTEXTUREIMAGE2DEXTPROC glTextureImage2DEXT;
	PFNGLTEXTUREIMAGE3DEXTPROC glTextureImage3DEXT;
	PFNGLTEXTUREPARAMETERFEXTPROC glTextureParameterfEXT;
	PFNGLTEXTUREPARAMETERFVEXTPROC glTextureParameterfvEXT;
	PFNGLTEXTUREPARAMETERIEXTPROC glTextureParameteriEXT;
	PFNGLTEXTUREPARAMETERIVEXTPROC glTextureParameterivEXT;
	PFNGLTEXTURESTORAGE1DEXTPROC glTextureStorage1DEXT;
	PFNGLTEXTURESTORAGE2DEXTPROC glTextureStorage2DEXT;
	PFNGLTEXTURESTORAGE3DEXTPROC glTextureStorage3DEXT;
	PFNGLTEXTURESUBIMAGE1DEXTPROC glTextureSubImage1DEXT;
	PFNGLTEXTURESUBIMAGE2DEXTPROC glTextureSubImage2DEXT;
	PFNGLTEXTURESUBIMAGE3DEXTPROC glTextureSubImage3DEXT;
	PFNGLUNMAPNAMEDBUFFEREXTPROC glUnmapNamedBufferEXT;
	// --

	// ++ wgl
	// --

	// ++ glx
	// --
};

#include "gl_hookset_defs.h"
